//
//	Includes
//
#include	"MOTTYPES.h"
#include	"Struct.h"
#include	"PortDef.h"
#include	"ModemReg.h"

//
//	Prototypes
//
#include	"ProtoType.h"

//
//	Variables
//
extern	UINT8	RtxMode;
extern	RxPacket_t	RxPacket;
extern	TxPacket_t	TxPacket;
extern	UINT8	RxDataBuffer[];
extern	UINT8	TxDataBuffer[];

//
//	Use external clock for fast operation
//
void UseExternalClock(void)
{
	ICGC1 = 0x50;         
	while (!ICGS1_ERCS); // Wait for external reference to be stable. 
	ICGC2_LOLRE = 1;
	ICGC2_MFD = 0x00;		// Mult factor of 4. 
}

//
//	Use internal clock for slow operation
//
void use_mcu_clock(void)
{
	ICGC1_RANGE = 1;	// Range High. 
	ICGC1_OSCSTEN = 1;	// Osc enabled in off mode. 
	ICGC1_CLKS = 0x00;	// Self clocked. 			
}

//
//	Restart the modem
//
void MODEM_restart()
{
 RtxMode = SYSTEM_RESET_MODE;
 IRQSC = 0x14; // Turn on the IRQ pin. 
 MODEM_RESET = 1; // Take modem out of reset 
 while (IRQSC_IRQF == 0) // Poll waiting for modem to assert the irq (i.e. ATTN). 
 {
 }
 (void)drv_read_spi_1(0x24);	// Clear modem interrupts 
 IRQSC |= 0x04;	// ACK the pending IRQ interrupt 
 IRQSC = 0x16; // Pin Enable, IE, IRQ CLR, negative edge. 
}

//
//	Reset (continuous) the modem
//
void MODEM_cont_reset()
{
	RtxMode = SYSTEM_RESET_MODE;
	IRQSC = 0x00; // Set for negative edge. 
	MODEM_RESET = 0; // Place the modem into reset 
}
//
//	Initialize the MCU COP, GPIO, SPI and IRQ
//
void CpuInit(void)
{
UINT16	irq_reg = 0;
UINT8 *ptr;	
	SOPT = 0x73;	// Turn off the watchdog. 	

  for(ptr = (UINT8*)0x100; ptr < (UINT8*)0x1000; *ptr = 0, ptr++);
  	
  TPM1MODH = 0;			// For 20MHz
  TPM1MODL = 250;  // 0.8uS * 250 = 200uS tick

  TPM1CNTH = 0;
  TPM1SC = 0x4b; // Timer divide by 8. (1uS tick for 8MHz .8uS for 10MHz clock).

  SRTISC = SRTISC&~0x07; // Disable wake up timer.
  SPMSC2 = SPMSC2&~0x03; // Enable deep sleep mode stop3.

	RTS_PORT = 1;		//	RTS port dir for output
	RTS_DATA = RTS_OFF;	//RTS_OFF;		//	Stop flow for begining
	CTS_PORT = 0;		//	CTS port dir for input

	LED1_OUT;	// Set LED1 port as outputs
	LED2_OUT;	// Set LED2 port as outputs
	LED3_OUT;	// Set LED3 port as outputs
	LED4_OUT;	// Set LED4 port as outputs
  
	LED1_OFF;	// Turn LED1 OFF
	LED2_OFF;	// Turn LED2 OFF
	LED3_OFF;	// Turn LED3 OFF
	LED4_OFF;	// Turn LED4 OFF

	PB1_PUE; 	// Push button1 pull up enable
	PB2_PUE;
	PB3_PUE;
	PB4_PUE;
}

