#include	"MOTTYPES.h"
#include	"Struct.h"
#include	"PortDef.h"
#include	"ModemReg.h"
#include	"ctype.h"
#include	"stdlib.h"
#include	"string.h"

//
//	Prototypes
//
#include	"ProtoType.h"

void LEDrun(UINT16);
void PacketInit(void);
void SciTxStart(void);
void SetupData(void);
void SetupMenu(void);
void LoadSetting(void);
void SaveSetting(void);
void FlashBlockRead(UINT16 bank, UINT8 offset, UINT8 *buffer, UINT16 size);
void FlashWrite(UINT16 bank, UINT8 offset, UINT8 *buffer, UINT16 size);


//
//	Variables
//
UINT8	RtxMode;	// Modem mode
UINT8	Channel;	// Freq channel
UINT8	BaudRate;	// Baud rate
UINT8	Power;		// Output power
UINT8	HostMode;	// Mode
UINT8	FlowFlag;	// Flow control
UINT8	Format; 	// Format
UINT8	TxSeqNum;	// TX packet seq number
UINT8	RxSeqNum;	// RX packet seq number
UINT8	RxLastNum;// RX packet seq number cache
UINT8	TickSave;	// Packet arrival time
UINT8	SizeSave;	// Arrived packet size
UINT8	MyID;			// My device ID
UINT8	TargetID;	// Target ID to be connected
UINT8	TimeOut;	// Error timeout count
UINT8	Connect;  // Connect flag

extern UINT8	RxRingBuf[];	// RX Ring buffer
extern volatile UINT16	RxRingOut, RxRingIn;	// RX Ring pointer
#define	RX_MASK		0x3ff
#define	RX_FULL		0x300
#define	RX_EMPTY	0x100

extern UINT8	TxRingBuf[];	// TX Ring buffer
extern volatile UINT16	TxRingOut, TxRingIn;	// TX Ring pointer
#define	TX_MASK		0x3ff
#define	TX_FULL		0x300
#define	TX_EMPTY	0x100

RxPacket_t	RxPacket;	// Packet control table for receive
TxPacket_t	TxPacket;	// Packet control table for transmit

PacketBuffer_t	RxPacketBuffer;	// Buffer for receive
PacketBuffer_t	TxPacketBuffer;	// Buffer for transmit

UINT8	AtBuffer[64];		// AT command buffer

volatile UINT8 TickCount;	// Timer tick for main loop
volatile UINT8 TimerCount;// Timer tick for delay

tJumpTable HookTable[25] @0x1035;

//
//	Timer Interrupt handler
//
interrupt void TimerInterrrupt(void)
{
	TPM1SC_TOF = 0; // Clear Timer int flag
	TickCount++;		// Count tick timer 	
	TimerCount++;		// Count tick timer 	
}

//
//	Setup PLL parameters
//
void SetupPll(void)
{
	ICGC1 = 0x78;     // Clock source is PLL    
	ICGC2 = 0x30;			// 20Mhz (2MHz * 10)
	while (!ICGS1_LOCK); // Wait for external reference to be stable. 
}

//
//	Main
//
void main(void)
{
INT16	temp;
//	Device init
	CpuInit();		// Port init
	ModemInit();	// Init modem registers
	SciInit();		// RS-232C init
	PacketInit();	// Buffer structure setup
	SetClockRate(3);	// Set 2Mhz clock out
	SetupPll();		// Set 20MHz PLL clock for 57600
	SCIBDL = 11;	// Set baudrate 56818 at 20MHz clock
  TPM1MODH = 0; // Set timer high value
  TPM1MODL = 250;  // 0.8uS * 250 = 200uS tick
	HookTable[7].adrs = TimerInterrrupt;	// Set vector 
	TimerPrescale(3);	// Set timeout 250kHz TickCount
	drv_write_spi_1(0x0b, 0x3e00); // GPIO 3,4,5,6,7 dirction is out 
  EnableInterrupts; // Enable interrupts
	LEDrun(3);		// Show power is ON
	if(PUSH_BUTTON2 == PRESSED)	// Setup mode?
		SetupMenu();	// Go setup menu (speed is 57600bps)
	LoadSetting();	// Load setting from flash

	if(BaudRate != 5)	// 57600bps?
	{									// No!
		SetClockRate(0);  // Set 16MHz clock out
		UseExternalClock();	// Switch clock sources from mcu to modem CLKO direct
	  TPM1MODH = 0;		 // Set timer high
	  TPM1MODL = 200;  // 1uS * 200 = 200uS tick
		switch(BaudRate)	// Case for each baud rate
		{
		case 0:	// 2400
			SCIBDL = 208;			// Set baudrate 2400 at 16MHz clock
			break;	
		case 1:	// 4800
			SCIBDL = 104;			// Set baudrate 4800 at 16MHz clock
			break;	
		case 2:	// 9600
			SCIBDL = 52;			// Set baudrate 9600 at 16MHz clock
			break;	
		case 3:	// 19200
			SCIBDL = 26;			// Set baudrate 19200 at 16MHz clock
			break;	
		case 4:	// 38400
			SCIBDL = 13;			// Set baudrate 38400 at 16MHz clock
			break;	
		}
	}

  if(Format)
    SCI1C1_PE = 1;

//	Initial setting	
  PowerAdjust(Power);	//Set power setting
	SetFreq(Channel);		//Select channel here (0-15)
  if(HostMode) LED4_ON; else LED4_OFF;	// Show Master or selector mode

//	PTCDD_PTCDD0 = 1; // DBG1
//	PTCDD_PTCDD1 = 1;	// DBG2

	switch(HostMode)	// Each mode
	{
	case 0:	// Device
		TargetID = 1;	// Target ID must be 1 (Master or Selector ID must be 1)
		break;
		
	case 1:	// Master
		MyID = 1;			// Force set my device ID = 1 (Master ID)
		TargetID = 2;	// Pair device ID is only 2 (1:1)
		break;

	case 2:	// Selector
		MyID = 1;		// Force set my device ID = 1 (Selector ID)
		break;
	}

#define	LOOPTIME	60		// 200uS * 60 = 12mS loop

WaitConnect:		// Modem command mode

	if(HostMode == 0)	// Device mode then wait until host requests connect
	{
		LED2_ON;			// Set error LED first
		for(;;)	// Wait connect loop
		{
			TickCount = 0;// Reset time
			RxSeqNum = 0;	// Clear RX packet seq num (will be set when recieve packet successfully)
			TickSave = 0;	// Clear receive timing
			RxPacket.status = TIMEOUT;	// Set TIMEOUT status first
			StartRecieve(0);		// Start recieve
			while(RtxMode != IDLE_MODE && TickCount < 254)// Wait recieve or timeout
				LOW_POWER_WHILE;		// Sleep
 			if(RtxMode != IDLE_MODE)	// Still recieving?
 			{
 				AbortReceive();	// Force to idle state
 			}
			else
			{
				if(RxPacketBuffer.DeviceID == MyID)	// Packet for me?
					break;	// Exit wait connect loop
			}
			while(TickCount < 255)	// Keep 50ms wait loop time
				LOW_POWER_WHILE;			// Sleep
		}
		RxEnable();
	}
	else if(HostMode == 1)		// Master mode?
	{
		RxEnable();
	}
	else if(HostMode == 2)		// Selector mode then AT modem simulation
	{
		RxEnable();
		for(;;)		// AT command parse loop
		{
			SciGets(AtBuffer, 125, 1);	// Get one line
			if(strncmp(AtBuffer, "ATD", 3) == 0 || strncmp(AtBuffer, "atd", 3) == 0)
			{		// ATD command?
				if(isdigit(AtBuffer[3]))	// ATD123 or ATDT123?
					TargetID = (UINT8)atoi(&AtBuffer[3]);	// Get number after ATD
				else
					TargetID = (UINT8)atoi(&AtBuffer[4]);	// Skip ATDT
				if(TargetID)	// Has value?
					break;			// Exit modem simulation loop
				SciPuts("\rBUSY\r\n");	// Show error
			}
			else
				SciPuts("\rOK\r\n");	// Null command response
		}
		RxDisable();
	}
// Start main loop
	TimeOut = 0;	// Set timeout count 0
	Connect = 0;	// Clear connect flag first
	LED2_ON;			// Set error LED first
	for(;;)
	{
		TickCount = 0;	// Reset time
//		DEB1ON;
		SetupData();		// Setup send packet
		SendData();			// Start transmit
		while(RtxMode != IDLE_MODE)	// Wait while sending
			LOW_POWER_WHILE;		// Sleep to save power
//		DEB1OFF;
		RxSeqNum = 0;	// Clear RX packet seq num (will be set when recieve packet successfully)
		TickSave = 0;	// Clear timing data
		RxPacket.status = TIMEOUT;		// Set TIMEOUT status first
		RxPacketBuffer.DeviceID = 0;	// Abort reqest?
//		DEB2ON;
		StartRecieve(5000);		// Start recieve (20ms timeout mode)
		while(RtxMode != IDLE_MODE && TickCount < (LOOPTIME - 1))// Wait for Recieve or timeout
			LOW_POWER_WHILE;		// Sleep
//		DEB2OFF;
 		if(RtxMode != IDLE_MODE)	// Still recieving?(no data)
 		{
			LED2_ON;		// Light error LED
			AbortReceive();	// Force to idle state
		}
		else	// Data has come
		{
			if(RxPacketBuffer.DeviceID == 255)	// Abort reqest from master/Selector?
				break;														// Abort sync loop
			if(RxPacketBuffer.DeviceID == MyID)	// Packet for me?
			{
				TimeOut = 0;	// Reset Timeout timer value
				if(HostMode == 2 && Connect == 0)	// Selector mode & first connect?
				{
					Connect = 1;										// Set connect flag
					SciPuts("\rCONNECT\r\n");				// Notify CONNECT message like AT modem
					RxEnable();
				}
			}
		}
		if(++TimeOut == 200)	// Timeout? (no response about 240ms)
			break;	// Exit sync loop
		temp = LOOPTIME;	// Set default main loop time
		if(HostMode)			// Only master & selector will adjust sync timing
		{	// Master will adjust timing to sync
			if(TickSave == 0)	// Can't recieve data
				temp = LOOPTIME + 5;	// Keep 12ms main loop + a moment to resync
			else if(RxPacketBuffer.WaitTime)	// Data has come?
				temp = (LOOPTIME * 3 / 2) - RxPacketBuffer.WaitTime + 3;// Adjust next transmit time
		}
		while(TickCount < temp)	// Keep 12ms main loop time
			LOW_POWER_WHILE;			// Sleep
	}
	RxDisable();
	if(HostMode == 2)	// Selector mode?
		SciPuts("\rNO CARRIER\r\nOK\r\n");	//Notify NO CARRIER as if it was AT modem
	goto WaitConnect;	// Go next connection
}

#define	MAXBUF	60		// MAX payload size

//
//	Setup Transmit data
//
void SetupData(void)
{
UINT8	index, getc;
//	RTS flow control
	if(RTS_DATA == RTS_OFF)	// Now RTS is off?
	{
		if(((RxRingIn - RxRingOut) & RX_MASK) < RX_EMPTY)	// RX buffer has space?&
		{
			RTS_DATA = RTS_ON;	// RTS ON
			LED3_ON;	// Show RTS is ON
		}
  }
	else
	{		// Now RTS is ON
		if(((RxRingIn - RxRingOut) & RX_MASK) > RX_FULL)	// RX buffer has no space or
		{
			RTS_DATA = RTS_OFF;	// RTS OFF
			LED3_OFF;	// SHow RTS is OFF
		}
	}
//	Check last packet sent successfully
	TxPacketBuffer.AckSeq = RxSeqNum;		// change received packet seq num only
	if(TickSave)	// Succsessfully recieved?
		TickSave = TickSave - (SizeSave >> 2);	// Get packet top timing
	TxPacketBuffer.WaitTime = TickSave;	// Set counter wait time (for sync time adjust)
	TxPacketBuffer.DeviceID = TargetID;	// Set device ID (for selector)
//
	if(TxPacketBuffer.Size)		// Not cleared?
		return;				// Resend same packet

//	Setup send data buffer
	for(index = 0;;)	// Data setup loop
	{
		if(RxRingIn == RxRingOut)	// Check RS-232C data has come?
			break;	// No data then exit loop
		getc = RxRingBuf[RxRingOut];	// Get char
		RxRingOut = ((RxRingOut + 1) & RX_MASK);	// Adjust pointer

		TxPacketBuffer.data[index]	= getc;	// Store char to packet buffer
		TxPacketBuffer.Size = ++index;	// Add length
		if(index == MAXBUF)	// Buffer full?
			break;	// Exit data setup loop
	}

	if(++TxSeqNum == 0)	// Increment Packet seq number fro new packet
		TxSeqNum++;		// If it's 0 then force to 1
	TxPacketBuffer.SeqNum = TxSeqNum;		// Set Send Seq num
	if(!(index & 1))	// Data size is even?
		index++;		// Add 1 byte for padding
	TxPacket.Length = index + 6;		// Must be odd number for correct transmit
}

//
//	Data indication callback
//
void DataIndication(void)
{
UINT8	i;
	if(RxPacket.status == SUCCESS)	// Receive status is SUCCESS?
	{
		LED1_XOR;			// Flip packet LED
		LED2_OFF;			// Clera error LED
		SizeSave = RxPacketBuffer.Size;	// Save current received packet size

		if(RxPacketBuffer.AckSeq == TxSeqNum)	// Ack seq matches sent seq num?
			TxPacketBuffer.Size = 0;	// Clear size for next packet send


		if((FlowFlag && CTS_DATA == CTS_OFF)	// Flow enable & CTS is off?
		|| ((TxRingIn - TxRingOut) & TX_MASK) >= TX_FULL)	// TX buffer has space?
		{		// No!
			RxSeqNum = 0; // Force retry if TX buffer is full
			return;		// Return (don't set data to SCI)
		}

		RxSeqNum = RxPacketBuffer.SeqNum; // Save recieved packet seq number
		if(RxSeqNum == RxLastNum)	// Already processed?
			return;			// return if it was duplicate packet sending
		RxLastNum = RxSeqNum;	// Save processed packet seq num

		if(RxPacketBuffer.Size)	// Received packet has payload?
		{
			for(i = 0; i < RxPacketBuffer.Size; i++)	// Send payload data to 232C
			{
				if(TxRingOut != ((TxRingIn + 1) & TX_MASK))	// Have space to put?
				{
					TxRingBuf[TxRingIn] = RxPacketBuffer.data[i];	// Put data to 232C send queue
					TxRingIn = (TxRingIn + 1) & TX_MASK;	// Increment pointer
				}
			}
			SciTxStart();	// Start automatic SCI transmition
		}
	}
	else
		LED2_ON;	// Show error
}

//
//	Packet arrive callback
//
void ArriveIndication(void)
{
	TickSave = TickCount;	// Save packet arrival time
}

//
//	Delay
// 
void Delay(UINT16 count)
{
UINT16	i;
	for (i = 0; i < count; i++)	// Repeat count
	{
		TimerCount = 0;		// Clear sub timer
		while(TimerCount < 8)	// Wait 8 tick count (1ms)
			LOW_POWER_WHILE;	// Sleep
	}
}

//
//	LED flash
//
void LEDrun(UINT16 j)
{
UINT16 i;
	for(i = 0; i < j; i++)	// Repeat Night Rider
	{
		LED1_ON;
		Delay(30);
		LED1_OFF; LED2_ON;
		Delay(30);
		LED2_OFF; LED3_ON;
		Delay(30);
		LED3_OFF; LED4_ON;
		Delay(30);
		LED4_OFF; LED3_ON;
		Delay(30);
		LED3_OFF; LED2_ON;
		Delay(30);
		LED2_OFF;
	}
}

//
//	Packet buffer init
//
void PacketInit(void)
{
	RxPacketBuffer.Size = 0;// Clear RX packet buffer size
	TxPacketBuffer.Size = 0;// Clear TX packet buffer size
	RxSeqNum = 0x80;	// Preset not to match first time compair
	RxLastNum = 0xff;	// Set dirty value to prevent first match
	TxSeqNum = 1;			// Set TX packet seq number
	TxPacket.Length = 0;	//Initialize the TX packet length
	TxPacket.data = (UINT8 *)&TxPacketBuffer; //Set the pointer to point to the TX buffer

	RxPacket.Length = 0;	//Initialize the RX packet length
	RxPacket.data = (UINT8 *)&RxPacketBuffer;//Set the pointer to point to the RX buffer
	RxPacket.status = 0;			//initialize status to 0.

	RtxMode = IDLE_MODE;	// Set modem mode is idle
}


const UINT8 BaudMsg[][6] = {	// Baud text
	"2400 ", "4800 ", "9600 ", "19200", "38400", "57600"};
const UINT8 ModeMsg[][9] = {	// Mode text
	"Device  ", "Master  ", "Selector"};
const UINT8 FormatMsg[][4] = {	// Mode text
	"8N1", "7E1"};

//
//	Setup config screen
//
void SetupScreen(void)
{
	SciPuts("\x1b[2J\x1b[1;1HSilentSystem SS-1 Setup");// Title
	SciPuts("\x1b[2;1HOperation mode[SPACE] = ");	// Mode
	SciPuts((UINT8*)ModeMsg[HostMode]);										// Mode text
	SciPuts("\x1b[3;1HChannel[LEFT,RIGHT] = ");		// Channel
	PutDec2(Channel + 11);												// Channel number (11 - 26)
	SciPuts(" (2.4");															// Show freq  (2.4
	SciPutc((UINT8) (Channel * 5 + 5) / 10 + '0');// Show freq  0
	SciPutc((Channel * 5 + 5) % 10 + '0');				// Show freq  5
	SciPuts("GHz)");															// Show uint  GHz)
	SciPuts("\x1b[4;1HBaud rate[UP,DOWN] = ");		// Show baud rate
	SciPuts((UINT8*)BaudMsg[BaudRate]);										// Show boud rate number
	SciPuts("\x1b[5;1HDeviceID[PageUp,PageDown] = ");// Show device ID
	PutDec2(MyID);																// Show My ID number (1-99)
	SciPuts("\x1b[6;1HPower[Home,End] = ");				// Show power
	PutDec2(Power);																// Show power number (0-15)
	SciPuts("\x1b[7;1HFlow control[f] = ");				// Show flow
	PutDec2(FlowFlag);														// Show flow number (0-1)
	SciPuts("\x1b[8;1HFormat[g] = ");		// Show baud rate
	SciPuts((UINT8*)FormatMsg[Format]);										// Show boud rate number
}

//
//	Setup Menu
//
void SetupMenu(void)
{
UINT8	ch;
	LoadSetting();	// Load setting from flash
	SetupScreen();  // Show config screen
	RxEnable();	// RTS on to allow PC to send data
	for(;;)					// Config loop
	{
		ch = SciGetc();// Get one char	
		if(ch == 0x1b || ch == 0x5b)	// ESC or ESC sequence char?
			continue;		 // Ignore it
		switch(ch)		 // Branch by input char
		{
		case ' ':	// Space?
			if(++HostMode == 3)	// Change mode
				HostMode = 0;
			break;

		case 'l':
		case 0x44:	// Left
			if(Channel)
				Channel--;	// Decrement freq
			break;
			
		case 'r':
		case 0x43:	// Right
			if(Channel < 15)
				Channel++;	// Increment freq.
			break;
			
		case 'u':
		case 0x41:	// Up
			if(BaudRate < 5)	// Increment baud rate
				BaudRate++;
			break;
			
		case 'd':
		case 0x42:	// Down
			if(BaudRate)
				BaudRate--;		// Decrement baud rate
			break;

		case '+':
		case 0x33:	// PageUP
			if(MyID < 99)
				MyID++;		// Increment My ID
			break;
			
		case '-':
		case 0x36:	// PageUP
			if(MyID > 1)
				MyID--;		// Decrement My ID
			break;
			
		case 'h':
		case 0x32:	// Home
			if(Power < 15)
				Power++;	// Increment power
			break;
			
		case 'e':
		case 0x35:	// End
			if(Power)
				Power--;	// Decrement power
			break;

		case 'f':	// f key?
			FlowFlag ^= 1;	// Toggle flow control
			break;

		case 'g':	// g key?
			Format ^= 1;	// Toggle flow control
			break;

		case 0x0d:	// Enter
			SaveSetting();	// Save setting data to flash
			SciPuts("\x1b[2J\x1b[1;1HReset SS-1 now");	// Show reset message
			RxDisable();
			for(;;)LOW_POWER_WHILE;	// Wait reset (hang up)
		}
		SetupScreen();	// Update config screen
	}
}

//
//	Load stored setting
//
void LoadSetting(void)
{
UINT8	buf[7];
	BufCopy(buf, (UINT8*)0x1600, 7);	// Read data from flash
	HostMode =	buf[0];	// Load mode
	Channel =		buf[1];	// Load channel
	BaudRate =	buf[2];	// Load baud rate
	MyID = 			buf[3];	// Load ID
	Power =			buf[4];	// Load Power 			
	FlowFlag =	buf[5];	// Load flow flag
	Format =	  buf[6];	// Load format
//	Range check
	if(HostMode > 3)	// Limit check
		HostMode = 0;		// Set device mode for default
	if(Channel > 15)	// Limit check
		Channel = 0;		// Set 2405 for default
	if(BaudRate > 5)	// Limit check
		BaudRate = 5; // Set 57600bps fro default
	if(MyID > 99)		// Limit check
		MyID = 1;			// Set master ID anyway
	if(Power > 15)	// Limit check
		Power = 15;	// Set upper limit
	if(HostMode == 0 && MyID < 2)// Device mode and My ID is wrong?
		MyID = 2;	// Device ID must be greater than 1
	if(HostMode)	// Master or Selector mode?
		MyID = 1;	// Device ID must be 0
	if(FlowFlag)	// Master or Selector mode?
#ifdef	CAM
		FlowFlag = 0;	// Set hardflow off
#else
		FlowFlag = 1;	// Set hard flow on
#endif
	if(Format > 1)	// 0 nor 1?
		Format = 0;	// Format must be 0 (8N1)
}

//
//	Load stored setting
//
void SaveSetting(void)
{
UINT8	buf[7];
	buf[0] = HostMode;	// Save mode
	buf[1] = Channel;		// Save freq
	buf[2] = BaudRate;	// Set baud rate
	buf[3] = MyID;			// Set My ID
	buf[4] = Power;			// Set power
	buf[5] = FlowFlag;	// Set padding for even number
	buf[6] = Format;	// Set padding for even number
	(void)NvmErase((UINT8*)0x1600);		// Erase flash first 
	(void)NvmWrite((UINT8*)0x1600, buf, 7);	// Write setting data to flash
}
