//
//	Includes
//
#include	"MOTTYPES.h"
#include	"Struct.h"
#include	"PortDef.h"
#include	"ModemReg.h"
#include	<string.h>
//
//	Prototypes
//
#include	"ProtoType.h"

//
//	Definitions
//
#define CLEAR_IRQ_FLAG		IRQSC_IRQACK = 1 //	Dependent upon interrupt source chosen
#define TX_IRQ_MASK				0x0040
#define RAMERR_IRQ_MASK		0x4000
#define RX_IRQ_MASK				0x0080
#define ATTN_IRQ_MASK			0x0400
#define CRC_VALID_MASK		0x0001
#define TIMER1_IRQ_MASK		0x0100
#define XCVR_SEQ_MASK			0xFFF8
#define LO_LOCK_IRQ_MASK	0x8000
#define CCA_IRQ_MASK			0x0020
#define DOZE_IRQ_MASK			0x0200
#define RESET_BIT_MASK		0x0080
//
#define AssertCE					MODEM_CE = 0 // Asserts the modem CE pin 
#define DeAssertCE				MODEM_CE = 1 // Deasserts the modem CE pin 
#define SPIClkInvert			SPIC1 |= 0x04; //Set CPHA bit of SPCR (clk polarity) 
#define SPIClkNormal			SPIC1 &= 0xFB; //clr CPHA bit of SPCR (clk polarity) 
//
//	Vars
//
UINT8		irq_value;

extern	UINT8	RtxMode;
extern	RxPacket_t	RxPacket;
extern	TxPacket_t	TxPacket;
extern tJumpTable HookTable[25] @0x1035;

#define WaitSPI_transfer_done() while(!SPI1S_SPRF);

//
//	modem initiated interrupt handler
//
interrupt void IrqHandler(void)
{
// The vector is defined in vectortable.c 
volatile UINT16 status_content, reg, temp; // Result of the status register read. 
UINT8	dataLength;		// Data length for the RX packet 
	DisableInterrupts;
	CLEAR_IRQ_FLAG; // Acknowledge the interrupt. modem IRQ pin still low.      
//	EnableInterrupts;
	dataLength = 0;		// Data length for the RX packet 
	status_content = drv_read_spi_1(STATUS_ADDR); // Read the modem status register. 
// Replace this with C code, if possible!!!!!!!!!!!!
// Test for IRQ glitch or possible fast double IRQ. 
	asm("BIH irq_high");
// IRQ is low. Re-read status. 
	CLEAR_IRQ_FLAG; // Acknowledge the interrupt. modem IRQ pin still low.      
	reg = drv_read_spi_1(STATUS_ADDR); // Read the modem status register. 
	status_content = status_content | reg; // Actual status is the OR of both. 
	asm("irq_high:");
	EnableInterrupts;
	status_content &= TX_IRQ_MASK | RX_IRQ_MASK | ATTN_IRQ_MASK | \
										CRC_VALID_MASK | TIMER1_IRQ_MASK | CCA_IRQ_MASK | \
										LO_LOCK_IRQ_MASK | DOZE_IRQ_MASK;
	if (RtxMode != RX_MODE_WTO)
	{
// If timeout not being used, mask out timer. 
		status_content &= ~(TIMER1_IRQ_MASK);
	}
	if ((status_content & ~(CRC_VALID_MASK)) == 0)
	{
// If no status bits are set just return. 
		return;
	}
// DOZE Complete Interrupt 
	if ((status_content & DOZE_IRQ_MASK) != 0)
	{
		drv_write_spi_1(T2_HI_ADDR, 0x8000); // disable the timer2 
		temp= drv_read_spi_1(MODE2_ADDR);
		temp &= 0xFFFC;				
		drv_write_spi_1(MODE2_ADDR, temp);      
		RtxMode = IDLE_MODE;
        
  }  
  // ATTN IRQ Handler
  if ((status_content & ATTN_IRQ_MASK) != 0)
  {
  // If attn interrupt, set the rtx_state mirror. 
  // For modem V2.x devices, read the reset indication in R25/7 first. 
  // If a reset is indicated, call back to a reset handler. 
		status_content = drv_read_spi_1(RESIND_ADDR); // Read the modem reset indicator register. 
		status_content &= RESET_BIT_MASK;
		if (status_content == 0) // Reset 
		{
			RtxMode = MODEM_RESET_MODE; // Set the rtx_state mirror to idle with attn. 
			return;
		}
		else
		{
			RtxMode = IDLE_MODE_ATTN; // Set the rtx_state mirror to idle with attn. 
			return;
		}
  }
  // TIMER1 IRQ Handler (Used for receiver timeout notification) 
  if ((status_content & TIMER1_IRQ_MASK) != 0) 
  {
  // If a timeout (and timeout enabled) is done, update the mode mirror state.   
	  	if (RtxMode == RX_MODE_WTO)
	  	{
			// Clear Timer1 if in RX_MODE_WTO 
			drv_write_spi_1(T1_HI_ADDR, 0x8000); // Disables TC1 and clears the IRQ. 
			drv_write_spi_1(T1_LO_ADDR, 0x0000);
			DeAssertRTXEN(); // Forces the modem to idle. 
			RxPacket.status = TIMEOUT;
// TIMEOUT notification 
			DataIndication();
			RtxMode = IDLE_MODE;
			return;
		}
	}
// LO LOCK IRQ - Occurs when modem loses LOCK 
// For receive cycles, re-enable. For transmit/CCA, abort. 
	if ((status_content & LO_LOCK_IRQ_MASK) != 0)
	{
		DeAssertRTXEN(); // Forces the modem to idle. 
		if((RtxMode == RX_MODE) || (RtxMode == RX_MODE_WTO) ||
			 (RtxMode == CCA_MODE) || (RtxMode == PULSE_TX_MODE)) // Unlock from receive cycles 
		{
			status_content = (drv_read_spi_1(MODE_ADDR) & 0xFF7F); // Read the modem trx register. Timer trigger off. 
			drv_write_spi_1(MODE_ADDR, status_content); // Re-write the trx register.   
	  	AssertRTXEN(); // Re-start the sequence. 
  	}
		else
			RtxMode = IDLE_MODE;
		return;
	}
	if ((RtxMode == IDLE_MODE) || ((status_content & CCA_IRQ_MASK) != 0) || ((status_content & TX_IRQ_MASK) != 0))
	{
// If in idle mode already or if CCA or TX is done, just return. 
		DeAssertRTXEN(); // Forces the modem to idle. 
		if(RtxMode == PULSE_TX_MODE)
		{
			status_content = (drv_read_spi_1(MODE_ADDR) & 0xFF7C); // Read the modem trx register. Timer trigger off. 
			drv_write_spi_1(MODE_ADDR, status_content | TX_MODE); // Re-write the trx register.   
	  	AssertRTXEN(); // Re-start the sequence. 
			return;
		}
		RtxMode = IDLE_MODE;
		return;
  }
  // If rx is done 
  if ((status_content & RX_IRQ_MASK) != 0)
  {  
		DeAssertRTXEN(); // Forces the modem to idle. 
		if ((status_content & CRC_VALID_MASK) == 0)
		{
				status_content = (drv_read_spi_1(MODE_ADDR) & 0xFF7C); // Read the modem trx register. Timer trigger off. 
				drv_write_spi_1(MODE_ADDR, status_content); // Clear rtx & timer register.   		
				DeAssertRTXEN();	// Forces the modem to idle. 
				RxPacket.status = CRCERROR;
				DataIndication();	// CRCERROR notification
				RtxMode = IDLE_MODE;
				return;
		}
		else
		{
			ArriveIndication();	// Packet arrival notification callback
			dataLength = (UINT8) (drv_read_spi_1(RX_PKT_LEN) & 0x7F); // Read received packet length register and mask off length bits 
			if (dataLength < 4) // Rx_pkt_length is bad when 0, 1 or 2. 3
			{
					status_content = (drv_read_spi_1(MODE_ADDR) & 0xFF7C); // Read the modem trx register. Timer trigger off. 
					drv_write_spi_1(MODE_ADDR, status_content); // Clear rtx & timer register.   		
					DeAssertRTXEN();	// Forces the modem to idle. 
					RxPacket.status = CRCERROR;
					DataIndication();	// CRCERROR notification
					RtxMode = IDLE_MODE;
					return;
			}

			// A valid packet has been received. 
			drv_write_spi_1(T1_HI_ADDR, 0x8000); // Disables TC1 and clears the IRQ. 
			drv_write_spi_1(T1_LO_ADDR, 0x0000);
			RxPacket.Length = dataLength;
			GetPacketBuffer();
			RxPacket.status = SUCCESS;
			DataIndication();	// Notify PHY that there is data available. 
			RtxMode = IDLE_MODE;       // set the rtx_state to idle 
			return;
		}
  }
}

//
//	Set Hibernate mode
//
void SetHibernateMode(void)
{
UINT16	temp;
	RtxMode = HIBERNATE_MODE;
	temp = drv_read_spi_1(MODE2_ADDR);	// 
	temp &= 0xFFFC;
	temp |= 0x0002;											// Hiberate enable 
	drv_write_spi_1(MODE2_ADDR, temp);	// 
}

//
//	Set Doze mode
//
void SetDozeMode(UINT32 Timeout)
{
UINT16	temp;
UINT32	time;
    
	RtxMode = DOZE_MODE;
	temp = drv_read_spi_1(MODE2_ADDR);		// 
	if(Timeout == 0)		// ACOMA mode, with clkout 
	{
		temp = drv_read_spi_1(IRQ_MASK);
		temp &= 0xFE0D;		// Change the IRQ_Mask to set Acoma en,
		temp |= 0x0100;		// doze irq disabled and tmr2 disabled
		drv_write_spi_1(IRQ_MASK, temp);      
		temp = drv_read_spi_1(MODE2_ADDR);	// 
		temp &= 0xFDFC;
		temp |= 0x0201;		// Doze (acoma) & CLKOUT enable 
		drv_write_spi_1(MODE2_ADDR, temp);	// 
	}
	else						 // DOZE mode with timeout, no clkout 
	{
		temp = drv_read_spi_1(IRQ_MASK);
		temp &= 0xFE0D;	// Change the IRQ_Mask to set Acoma dis,
		temp |= 0x0012;	// doze enabled and tmr2 enabled
		drv_write_spi_1(IRQ_MASK, temp);      
        
		time = GetTimeRequest();
		Timeout += time;
		drv_write_spi_1(T2_HI_ADDR, (((UINT16)(Timeout >> 16)) & 0x00FF)); 
		drv_write_spi_1(T2_LO_ADDR, ((UINT16)Timeout));  // Enable the Timer 2 
                                                         
		temp = drv_read_spi_1(MODE2_ADDR);
		temp &= 0xFDFC;		// Disable CLKOinDozeand 
		temp |= 0x0001;		// enter in Doze mode 
		drv_write_spi_1(MODE2_ADDR, temp);      
	}
}

//
//	Wake the modem from Hibernate/Doze mode
//
void WakeModem(void)
{
	MODEM_ATTN = 0; // Assert ATTN 
	MODEM_ATTN = 1; // Deassert ATTN 
}

//
//	Wake the modem from Hibernate or Doze.
//
void WakeRequest(void)
{
UINT16 temp;
    
    WakeModem();												// Wake up the device 
    while (RtxMode != IDLE_MODE_ATTN)
			LOW_POWER_WHILE;										// Wait until ATTN 
    temp = drv_read_spi_1(MODE2_ADDR);		// 
                                                     
		temp &= 0xFFFC;												// Hiberate and Doze disable 
		drv_write_spi_1(MODE2_ADDR, temp);		// 
                                                     
    drv_write_spi_1(T2_HI_ADDR, 0x0000);	// Disable Timer2
                                                     
		RtxMode = IDLE_MODE;
}

//
//	Deassert the modem RTXEN pin (forces IC to idle)
//
void DeAssertRTXEN(void)
{
	MODEM_RTXEN = 0; // Deassert RTXEN 
}

//
//	Assert the modem RTXEN pin (initiates programmed cycle)
//
void AssertRTXEN(void)
{
	MODEM_RTXEN = 1; // Assert RTXEN 
}

//
//	disable modem interrupts
//
#define	DisableModemInterrupts()	{irq_value=MODEM_IRQ_SOURCE;MODEM_IRQ_SOURCE=irq_value & ~(0x06);}
//	irq_value = MODEM_IRQ_SOURCE;	// Save the context of the MODEM_INT_REG (global) 
//	MODEM_IRQ_SOURCE = irq_value & ~(0x06);  // Disable the modem interrupt source 

//
//	restore modem interrupts to previous condition
//

#define	RestoreModemInterrupts()	{MODEM_IRQ_SOURCE = irq_value;}
//	MODEM_IRQ_SOURCE = irq_value;	// Restore the context of the IRQ register from global 

//
//	Write to SPI
//
void drv_write_spi_1(UINT8 addr, UINT16 content)
{
UINT8 temp; // Used to flush the SPI1D register during read 
	temp = SPI1S; // Clear status register (possible SPRF, SPTEF)   
  temp = SPI1D; // Clear receive data register. SPI entirely ready for read or write                        
  DisableModemInterrupts(); // Necessary to prevent double SPI access 
  AssertCE; // Enables modem SPI 
  SPI1D = addr & 0x3F; // Mask address, 6bit addr. Set write bit (i.e. 0). 
  WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set. Now write content MSB 
  temp = SPI1D; // Clear receive data register. SPI entirely ready for read or write 
  SPI1D = content >> 8; // Write MSB 
  WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set. Now write content LSB 
  temp = SPI1D; // Clear receive data register. SPI entirely ready for read or write 
  SPI1D = content & 0x00FF; // Write LSB 
  WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set. Now read last of garbage 
  temp = SPI1D; // Clear receive data register. SPI entirely ready for read or write 
  DeAssertCE; // Disables modem SPI 
  RestoreModemInterrupts(); // Restore modem interrupt status 
}

//
//	Read word from SPI
//
UINT16 drv_read_spi_1(UINT8 addr)
{
UINT16  w; // w[0] is MSB, w[1] is LSB 
UINT8 temp; // Used to flush the SPI1D register during read 
	temp = SPI1S; // Clear status register (possible SPRF, SPTEF)   
  temp = SPI1D; // Clear receive data register. SPI entirely ready for read or write                        
  DisableModemInterrupts(); // Necessary to prevent double SPI access 
  AssertCE; // Enables modem SPI 
  SPI1D = (addr & 0x3f) | 0x80; // Mask address, 6bit addr, Set read bit. 
  WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set 
  temp = SPI1D; // Clear receive data register. SPI entirely ready for read or write 
  SPI1D = addr; // Dummy write. Receive register of SPI will contain MSB 
  WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set. Get MSB 
  ((UINT8*)&w)[0] = SPI1D; // MSB 
  SPI1D = addr; // Dummy write. Waiting until after reading received data insures no overrun 
  WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set. Get LSB 
  ((UINT8*)&w)[1] = SPI1D; // LSB 
  DeAssertCE; // Disables modem SPI 
  RestoreModemInterrupts(); // Restore modem interrupt status 
  return w;
}

//
//	Read second word from SPI
//
UINT16 drv_read_spi_2(UINT8 addr)
{
UINT16  w; // w[0] is MSB, w[1] is LSB 
UINT8 temp; // Used to flush the SPI1D register during read 
	temp = SPI1S; // Clear status register (possible SPRF, SPTEF)   
  temp = SPI1D; // Clear receive data register. SPI entirely ready for read or write                        
  DisableModemInterrupts(); // Necessary to prevent double SPI access 
  AssertCE; // Enables modem SPI 
  SPI1D = (addr & 0x3f) | 0x80; // Mask address, 6bit addr, Set read bit. 
  WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set 
  temp = SPI1D; // Clear receive data register. SPI entirely ready for read or write 
  SPI1D = addr; // Dummy write. Receive register of SPI will contain MSB 
  WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set. Get MSB 
  ((UINT8*)&w)[0] = SPI1D; // MSB 
  SPI1D = addr; // Dummy write. Waiting until after reading received data insures no overrun 
  WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set. Get LSB 
  ((UINT8*)&w)[1] = SPI1D; // LSB 
  SPI1D = addr; // Dummy write. Waiting until after reading received data insures no overrun 
  WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set. Get LSB 
	((UINT8*)&w)[0] = SPI1D;  // MSB
  SPI1D = addr; // Dummy write. Waiting until after reading received data insures no overrun 
  WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set. Get LSB 
  ((UINT8*)&w)[1] = SPI1D; // LSB 
	DeAssertCE;			// Disables modem SPI
  RestoreModemInterrupts(); // Restore modem interrupt status 
  return w;
}



//
//	Set modem clock rate
//	0:16MHz, 1:8MHz, 2:4MHz, 3:2MHz, 4:1Mhz, 5:62.5KHz, 6:32.768kHz, 7:16.393kHz
//
void SetClockRate(UINT8 freq)
{
volatile UINT16 temp;
	temp = drv_read_spi_1(CLKS_ADDR); // Read register and re-write
	temp &= 0xFFF8;
	temp |= freq;
	drv_write_spi_1(CLKS_ADDR, temp);
}
//
//	Set TX Power
//	0 <-> 15
//
void PowerAdjust(UINT8 power)
{
UINT16	regbuf;
	if(power > 15)	power = 15;
	regbuf = drv_read_spi_1(PA_ADJUST_ADDR) & 0xff00;
	regbuf |= (power << 4) | 0x0c;	// 0xc is chip default
	drv_write_spi_1(PA_ADJUST_ADDR, regbuf);
}
//
//	Set frequency
//
void SetFreq(UINT8 freq)
{
UINT16	temp;
	if(freq > 15) freq = 8;	// Set Default
	temp = freq * 5 + 5;
	drv_write_spi_1(LO1_IDIV_ADDR, 0x0f95 + (temp >> 4));
	drv_write_spi_1(LO1_NUM_ADDR, temp << 12);
}

//
//	CCA detect
//
UINT8 CCADetect(void)
{
UINT16	temp;
	RtxMode = CCA_MODE;                // Write energy detect mode 
	temp = drv_read_spi_1(MODE_ADDR);
	temp &= 0xFFcc;
	temp |= CCA_MODE | 0x10;	// 0x10 means CCA mode
	drv_write_spi_1(MODE_ADDR, temp);
	AssertRTXEN();
	while(RtxMode != IDLE_MODE)       // Wait for energy detect to complete 
		LOW_POWER_WHILE;
	return drv_read_spi_1(CCA_RESULT_ADDR) >> 8;
}

//
//	Energy detect
//
UINT8 EnergyDetect(void)
{
UINT16	temp;
	RtxMode = CCA_MODE;                // Write energy detect mode 
	temp = drv_read_spi_1(MODE_ADDR);
	temp &= 0xFFcc;
	temp |= CCA_MODE | 0x20;	// 0x20 means ED mode
	drv_write_spi_1(MODE_ADDR, temp);
	AssertRTXEN();
	while(RtxMode != IDLE_MODE)       // Wait for energy detect to complete 
		LOW_POWER_WHILE;
	return drv_read_spi_1(CCA_RESULT_ADDR) >> 8;
}

//
//	Get Link quality of last packet recieve
//
UINT8 ReadStrength(void)
{
	return drv_read_spi_1(CCA_RESULT_ADDR) >> 8;
}

//
//	Set timer prescaler
//
void TimerPrescale(UINT8 freq) 
{
volatile UINT16	temp;
		if(freq > 7) freq = 3;	// Default (250kHz)
    
    temp = drv_read_spi_1(PRESCALE_ADDR);
    temp &= 0xFFF8;
    temp |= freq;
    drv_write_spi_1(PRESCALE_ADDR, temp);
}

//
//	SetTimerValue
//	(PLMESetTimeRequest)
//	Set modem timer value (i.e. initialize).
//
void SetTimerValue(UINT32 value)
{
UINT16 HiWord, LoWord;
UINT16 temp;
    
// Split 32 bit input into 2 16 bit values
		HiWord = (UINT16) (value >> 16) & 0x000000FF;
		LoWord = (UINT16) value & 0x0000FFFF;
// Program Time1 comparator with the desired value    
		drv_write_spi_1(T1_HI_ADDR, HiWord);
		drv_write_spi_1(T1_LO_ADDR, LoWord);
// Get current state of the MODE2 modem register
		temp = drv_read_spi_1(MODE2_ADDR);
// Set the Tmr_load bit 
		temp |= 0x8000;
// Now write the value back to modem register MODE2
		drv_write_spi_1(MODE2_ADDR, temp);
// Clear the tmr_load bit
		temp &= 0x7FFF;
// Clr the tmr_load bit to prepare for next set_time_request. 
    drv_write_spi_1(MODE2_ADDR, temp);    
    return;
}
//
//	Get Timer value
//
UINT32 GetTimeRequest(void)
{
UINT32 UpperWord, LowerWord;
UINT32 CurrentTime;
    
    DisableInterrupts;
    UpperWord = drv_read_spi_1(TIMESTAMP_HI_ADDR);
    LowerWord = drv_read_spi_1(TIMESTAMP_LO_ADDR);
    UpperWord &= TIMESTAMP_HI_MASK;              /* Clears TS_HELD bit. */
    CurrentTime = (UINT32) (UpperWord << 16) | LowerWord;
    EnableInterrupts;
    return CurrentTime;
}

//
//	Enable Timer1
//	(PLMEEnablemodemTimer1)
//	Set modem timer compare value.
//
void EnableTimer1(UINT32 value)
{
UINT16 HiWord, LoWord;
//	Load the timeout value into T1 with Timer disabled.	
		HiWord = (UINT16) (value >> 16) & 0x000000FF;
		LoWord = (UINT16) value & 0x0000FFFF;

    drv_write_spi_1(T1_HI_ADDR, HiWord | 0x8000);
    drv_write_spi_1(T1_LO_ADDR, LoWord);
//	Turn Timer1 mask on
    drv_write_spi_1(T1_HI_ADDR, HiWord);
    drv_write_spi_1(T1_LO_ADDR, LoWord);
}

//
//	Disable Timer1
//	(PLMEDisablemodemTimer1
//	Disable modem timer comparator TC1.
void DisableTimer1(void)
{
// Load the timeout value into T1 with Timer disabled
// Clear Timer1 if in RX_MODE_WTO
    drv_write_spi_1(T1_HI_ADDR, 0x8000);
    drv_write_spi_1(T1_LO_ADDR, 0x0000);
//	irq_mask_reg = drv_read_spi_1(IRQ_MASK);      // NOT USED 
//	irq_mask_reg &= ~TIMER1_IRQMASK_BIT;
//	drv_write_spi_1(IRQ_MASK, irq_mask_reg);
}


//
//	StateRequest
//	(PLMESetTrxStateRequest)
//	Set the modem transceive operation.
//
void StateRequest(UINT8 mode)
{
UINT16 temp;

	DeAssertRTXEN(); 
	temp = drv_read_spi_1(MODE_ADDR);
	temp &= 0xFFFC;		// Clear mode bit
	switch(mode)
	{
	case IDLE_MODE:	// IDOL
//	temp |= IDLE_MODE;
		RtxMode = IDLE_MODE;
		drv_write_spi_1(MODE_ADDR, temp);
		break;

	case RX_MODE:	// RX
		RtxMode = RX_MODE;
		temp |= RX_MODE;
		drv_write_spi_1(MODE_ADDR, temp);
		AssertRTXEN(); 
		break;

	case RX_MODE_WTO:	// RX with timer
		RtxMode = RX_MODE_WTO;
		temp |= RX_MODE;
		drv_write_spi_1(MODE_ADDR, temp);
		AssertRTXEN(); 
		break;

	case TX_MODE:		// TX
		RtxMode = TX_MODE;
		temp |= TX_MODE;
		drv_write_spi_1(MODE_ADDR, temp);
		AssertRTXEN(); 
		break;

	case PULSE_TX_MODE: // Write Tx. Note: force LO lock not used 
		RtxMode = PULSE_TX_MODE;	
		temp |= TX_MODE;
		drv_write_spi_1(MODE_ADDR, temp);
		AssertRTXEN(); 
		break;

	default:
		RtxMode = IDLE_MODE;	
  	break;	// Do nothing
    }
}

//
//	Store packet data and start transmition
//
void SendData(void)
{
	SetPacketBuffer();	//	Store data into packet RAM
	StartTransmit();
}

//
//	Start transmition
//
void StartTransmit(void)
{
	while(RtxMode != IDLE_MODE)
		LOW_POWER_WHILE;
	StateRequest(TX_MODE);
}

//
//	Set RX or TX state
//	(MLMERXEnableRequest)
//
void StartRecieve(UINT32 timeout)
{
	if (timeout == 0)				// Timeout disabled?
		StateRequest(RX_MODE);
	else
	{
		SetTimerValue(0x00000000);
		EnableTimer1(timeout);
		StateRequest(RX_MODE_WTO);
	}
}

//
//	AbortReceive
//	(MLMERXDisableRequest)
//
void AbortReceive(void)
{
	DisableTimer1();
	StateRequest(IDLE_MODE);
}

//
//	Send Packet data into modem RAM
//
void SetPacketBuffer(void)
{
UINT8	i, ibyte, temp; // i, ibyte are counters. temp is used to flush the SPI1D register during read 
UINT16	reg; // TX packet length register value 
	reg = drv_read_spi_1(TX_PKT_LEN); // Read the TX packet length register contents 
	reg = (0xFF80 & reg) | (TxPacket.Length + 2); // Mask out old length setting and update. Add 2 for CRC 
	drv_write_spi_1(TX_PKT_LEN, reg); // Update the TX packet length field 
	temp = SPI1S; // Clear status register (possible SPRF, SPTEF)   
	temp = SPI1D; // Clear receive data register. SPI entirely ready for read or write                        
	DisableModemInterrupts(); // Necessary to prevent double SPI access 
	AssertCE; // Enables modem SPI 
	SPI1D = TX_PKT; // SPI TX ram data register 
	WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set. Now write content MSB 
	temp = SPI1D; // Clear receive data register. SPI entirely ready for read or write 
	ibyte = 0; // Byte counter for *contents 
	for (i = 0; i < ((TxPacket.Length + 1) >> 1); i++) // Word loop. Round up.  
	{
		SPI1D = TxPacket.data[ibyte + 1];	// Write MSB 
    SPI1D = 0;
		WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set. Now write content LSB 
		temp = SPI1D; // Clear receive data register. SPI entirely ready for read or write 
		SPI1D = TxPacket.data[ibyte];	// Write LSB   		
    SPI1D = 0;
		ibyte = ibyte + 2; // Increment byte counter 
		WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set.
		temp = SPI1D; // Clear receive data register. SPI entirely ready for read or write 
	}
	DeAssertCE; // Disables modem SPI 
	RestoreModemInterrupts(); // Restore modem interrupt status 
}

//
//	Get Packet data from modem RAM
//
void GetPacketBuffer(void)
{
UINT8	i, temp, ibyte;
UINT16	length;
	length = drv_read_spi_1(RX_PKT_LEN) & 0x7f;
	if(length >= 3)
		RxPacket.Length = length - 2;
	else
		RxPacket.Length = 0;
        
	if((RxPacket.Length >= 1) && \
        (RxPacket.Length <= 125))
	{
		temp = SPI1S; // Clear status register (possible SPRF, SPTEF)  
		temp = SPI1D; // Clear data register. SPI entirely ready for read or write                       
		DisableModemInterrupts(); // Necessary to prevent double SPI access
		AssertCE;                 // Enables modem SPI
	 	SPI1D = RX_PKT | 0x80; // SPI RX ram data register 
  	WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set.
  	temp = SPI1D; // Clear receive data register. SPI entirely ready for read or write 
		SPI1D = temp; // Dummy write. Receive register of SPI will contain MSB garbage for first read 
		WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set.
		temp = SPI1D; // Clear receive data register. SPI entirely ready for read or write 
		SPI1D = temp; // Dummy write. Receive register of SPI will contain LSB garbage for first read 
		WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set.
		temp = SPI1D; // Clear receive data register. SPI entirely ready for read or write 
		ibyte = 0; // Byte counter for *contents 
		for (i = 0; i < ((length - 1) >> 1); i++) // Word loop. Round up. Deduct CRC. 
		{
			SPI1D = temp; // Dummy write. Receive register of SPI will contain MSB 
			WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set. Get MSB 
			if ((ibyte + 3) == length) // For a trailing garbage byte, just read and discard 
				temp = SPI1D; // Discard 
			else
				RxPacket.data[ibyte + 1] = SPI1D; // Read MSB 
			SPI1D = temp; // Dummy write. Receive register of SPI will contain LSB 
			WaitSPI_transfer_done(); // For this bit to be set, SPTED MUST be set. Get LSB 
			RxPacket.data[ibyte] = SPI1D; // Read LSB 
	 		ibyte=ibyte + 2; // Increment byte counter 
	 	}
 		DeAssertCE; // Disables modem SPI 
 		RxPacket.status = SUCCESS;
		RestoreModemInterrupts(); // Restore modem interrupt status 
  }
	// Check to see if a larger packet than desired is received.   
  if (RxPacket.Length > 125)
	  RxPacket.status = OVERFLOW;
}

//
//	Get Chipset Version
//
UINT8	GetVersion(void)
{
	return (UINT8)((drv_read_spi_1(VERSION_REG) & VERSION_MASK) >> 10);
}


//
//	Modem init
//
void ModemInit(void)
{
// modem v2.2 register settings 
	drv_write_spi_1(0x1B,0x8000); // Disable TC1. 
	drv_write_spi_1(0x1D,0x8000); // Disable TC2. 
	drv_write_spi_1(0x1F,0x8000); // Disable TC3. 
	drv_write_spi_1(0x21,0x8000); // Disable TC4. 

	drv_write_spi_1(0x07,0x5E00); // Enable CLKo in Doze 

	drv_write_spi_1(0x0C,0x0300); // IRQ pull-up disable. 
	(void)drv_read_spi_1(0x25); // Sets the reset indicator bit 
	drv_write_spi_1(0x04,0xA08D); // LR ADDED New cal value 
	drv_write_spi_1(0x08,0xFFF7); // Preferred injection 
	drv_write_spi_1(0x05,0x8351); // Acoma, TC1, Doze, ATTN masks, LO1, CRC 
	drv_write_spi_1(0x06,0x4720); // CCA, TX, RX, energy detect 
 
 // Read the status register to clear any undesired IRQs. 
	(void)drv_read_spi_1(0x24); // Clear the status register, if set 
	RtxMode = IDLE_MODE; // Update global to reflect modem status 
	HookTable[1].adrs = IrqHandler;	// Set vector 
}
